
pro remove_tmpglt, glt_fid, glt_name

  ; This utility is used by omega_map_projection to clean up temporary GLT files.

  if (n_elements(glt_fid) gt 0) then begin
    fid = glt_fid[0]
    if (fid ge 0) then begin
      ; Remove GLT from available bands list:
      envi_file_mng,id=fid,/remove,/delete
    endif
  endif

  ; And delete the temp glt header, either file.img.hdr or file.hdr:
  if (strtrim(glt_name[0]) ne '') then begin
    gltbn = strtrim(file_basename(glt_name[0]),2)
    p = strpos(gltbn,'.',/reverse_search)
    if (p gt 0) then gltbn=strmid(gltbn,0,p)
    gltbn = file_dirname(glt_name[0], /mark_directory) + gltbn
    glt_hdr = file_search(gltbn+'*.hdr',/fold_case)
    k = where((glt_hdr ne ''),nk)
    if (nk gt 0) then begin
      file_delete,glt_hdr[k], /quiet,/noexpand_path
    endif
  endif

  return
end


;******************************************************************************
;
; function omega_map_projection, proj_info
;
; Map project OMEGA data. Based on crism_projection.pro in the CRISM Analysis Tool.
;
; INPUT:
; proj_info  Projection inputs:
;   proj_info.image.dims       Image dimensions
;   proj_info.image.fid        Image file ID
;   proj_info.image.bands      Image band indices
;   proj_info.output.outfile   Output file name
;   proj_info.output.pixsize   Pixel size for output, m
;   proj_info.nav.fid          NAV Cube File ID
;   proj_info.nav.lat_band     Band of NAV Cube with latitude
;   proj_info.nav.lon_band     Band of NAV Cube with longitude
;   proj_info.nav.ema_band     Band of NAV Cube with emission angle
;
; OUTPUT:
; Parameters:  none
; Return value:  Status structure-
;   val: 0, OK; -1, fatal failure; 1, nonfatal problem
;   msg: string containing error message
;   
; KEYWORDS: none
;
; NOTES:
;	This routine performs the projection work in a batchable manner.
;	The projection is consistent with the CRISM MTRDR projection in terms of
;	latitude of origin, datum, and names.
;
;******************************************************************************

function omega_map_projection,  proj_info

  COMPILE_OPT IDL2
  e = ENVI(/CURRENT)

  ; Planet parameters
  MARS_RE = 3396.190    ; equatorial radius, km
  MARS_RP = 3376.200    ; polar radius, km


  projstat = {status: 0, msg: 'OK'}     ; default status structure

  ; -------------   Input Projection  -------------
  ; Create INPUT Geographic projection structure to associate w/file
  ;  (as is, file has an "Arbitrary" projection, which cannot be used
  ;  to create a GLT)
  projin = envi_proj_create(/geographic)


  ; ---------------   Output Projection  ---------------

  ; Get coordinates and emission angles from the NAV Cube:
  raster = ENVIFIDToRaster(proj_info.nav.fid)
  ema = raster.GetData(BANDS=proj_info.nav.ema_band, SUB_RECT=[0,0,proj_info.image.dims[2], proj_info.image.dims[4]])
  lat = raster.GetData(BANDS=proj_info.nav.lat_band, SUB_RECT=[0,0,proj_info.image.dims[2], proj_info.image.dims[4]])
  lon = raster.GetData(BANDS=proj_info.nav.lon_band, SUB_RECT=[0,0,proj_info.image.dims[2], proj_info.image.dims[4]])

  ; Reference coordinates:
  coordstat = omega_reference_coordinates(lat, lon, ema, reflat, reflon, min_ema_row)
  if (coordstat ne 0) then begin
    ; That failed, just use median lat/lon
    reflat = median(lat)
    reflon = median(lon)
  endif
  
  ;print, "Ref location selected as", reflat, reflon

  ; Origin of projection. Datum, projection parameters.
  lonorigin = round(reflon*100.0) / 100.0     ; round to 2 decimal places
  if (abs(reflat) le 65.0) then begin
    ; The following code snippet to determine the 5-degree rolling origin latitude
    ; follows F. Seelos in mro_crism_mars_projections:
    if (reflat ge 0.0) then begin
      ; At northern latitudes, largest multiple of 5 < latitude:
      latorigin = floor(reflat/5.0) * 5.0
    endif else begin
      ; Southern latitudes...
      latorigin = ceil(reflat/5.0) * 5.0
    endelse
    ; Choose a datum:
    ilo = round(abs(latorigin))
    proj_gcs = 'GCS_Mars_2000_IAU_IAG_custom_sphere_lat' + string(ilo,format='(I2.2)')
    proj_datum = 'D_Mars_2000_IAU_IAG_custom_sphere_lat' + string(ilo,format='(I2.2)')
    proj_spheroid = 'Mars_2000_IAU_IAG_custom_sphere_lat' + string(ilo,format='(I2.2)')
    s = map_pe_get_option_table(1, datum_struct)
    datum_ok = 1
    if (s gt 0) then begin
      datum_list = strupcase(strtrim(datum_struct.name,2))
      idx = where(datum_list eq strupcase(strtrim(proj_datum,2)))
      if (idx[0] lt 0) then datum_ok = 0
    endif else begin
      datum_ok = 0
    endelse
    if (~datum_ok) then begin
      ; Fall back on a default datum:
      msg = 'MRO custom datum ' + proj_datum + ' not available. Projection may not match MTRDR. Using'
      projstat = {status: 1, msg: msg}
      proj_datum = ''
    endif
    ; Define an equirectangular projection:
    ;  Assumes spherical body, needs radius in meters,
    ;  lat and lon of projection origin and false easting and
    ;  northing (in meters). Radius is radius at projection origin.
    lorad = latorigin * !PI/180.0
    rad = 1000 * MARS_RE * MARS_RP / sqrt( (MARS_RP*cos(lorad))^2 + (MARS_RE*sin(lorad))^2 )
    type = 17
    proj_params = [rad, latorigin, lonorigin, 0.0, 0.0]
    ;name = 'Mars Sphere-Based Equirectangular'
    ; Projection name following F. Seelos in mro_crism_mars_projections:
    proj_name = 'MRO Mars Equirectangular [IAU 2000] [' + strtrim(string(latorigin, format = '(f6.2)'),2) + 'N' + '; ' + strtrim(string(lonorigin, format = '(f7.2)'),2) + 'E' + ']'

    projstr = 'PROJCS["' + proj_name + '", GEOGCS["' + proj_gcs + '", DATUM["' + proj_datum + '", SPHEROID["' + proj_spheroid + '",'  + strtrim(string(proj_params[0]),2) + ',0.0]],' $
      + 'PRIMEM["Reference_Meridian",0.0], UNIT["Degree",0.0174532925199433]],' $
      + 'PROJECTION["Equidistant_Cylindrical"], PARAMETER["False_Easting",0.0], PARAMETER["False_Northing",0.0],' $
      + 'PARAMETER["Central_Meridian",'+ strtrim(string(proj_params[2]),2) + '],PARAMETER["Standard_Parallel_1",' + strtrim(string(proj_params[1]),2) + '],UNIT["Meter",1.0]]'
  endif else begin
    ; Polar stereographic. Parameters are semimajor and semiminor ellipsoid in meters,
    ; lat and lon of projection origin, and false easting and northing in meters:
    type = 31
    if (reflat lt 0.0) then begin
      latorigin = -90.0
      proj_name = 'Mars South Polar Stereographic [IAU 2000] '
      proj_params = [MARS_RE * 1000, MARS_RP * 1000, latorigin, lonorigin, 0.0, 0.0]
      proj_datum = 'D_Mars_2000'
      pole = 'S'
    endif else begin
      latorigin = 90.0
      proj_name = 'Mars North Polar Stereographic [IAU 2000] '
      proj_params = [MARS_RE * 1000, MARS_RP * 1000, latorigin, lonorigin, 0.0, 0.0]
      proj_datum = 'D_Mars_2000'
      pole = 'N'
    endelse
    revers_f = proj_params[0] /(proj_params[0] -proj_params[1])

    ;;GCS_Mars_2000_Sphere
    ;;;SPHEROID[ "Mars_2000",  3396190.0,  169.8944472236118]
    ;;;The values are Semi-Major axis length in Meters, Inverse Flattening (1 / f)  so 169.8944472236118 = 3396190 /(3396190-3376200)

    projstr = 'PROJCS["' + proj_name + '", GEOGCS["GCS_Mars_2000", DATUM["D_Mars_2000", SPHEROID["MARS_2000_IAU_IAG",' + strtrim(string(proj_params[0]),2) + ',' + strtrim(string(revers_f),2) + ']],' $
      + 'PRIMEM["Reference_Meridian",0.0], UNIT["Degree",0.0174532925199433]],' $
      + 'PROJECTION["Stereographic"], PARAMETER["False_Easting",0.0], PARAMETER["False_Northing",0.0],' $
      + 'PARAMETER["Central_Meridian",'+ strtrim(string(proj_params[3]),2) + '], PARAMETER["Scale_Factor",1.0], PARAMETER["Latitude_Of_Origin",' + strtrim(string(proj_params[2]),2) + '],UNIT["Meter",1.0]]'

  endelse

  ; Pixel size in meters
  pixel_size = proj_info.output.pixsize

  ; Finally  - output projection:
  projout = envi_proj_create(name=proj_name, type=type, params=proj_params, datum=proj_datum)

  ; ---------------   Project The Data   ----------------

  ; Create the look up table:
  
  ; a (reasonably) foolproof temp file name: only assumes this next line isn't
  ; run in two instances during the same second according to the system clock
  time_str = strtrim(long(systime(/seconds)),2)
  tmp_glt_name = filepath('glt_proj_'+time_str+'.tmp',/tmp)
  
  ; TODO: investigate whether ENVI_GLT_DOIT's in_memory keyword might work, too
  ENVI_DOIT, 'ENVI_GLT_DOIT', DIMS=dims, I_PROJ=projin, O_PROJ=projout, $
    out_name=tmp_glt_name,r_fid=glt_fid, ROTATION=0, pixel_size=pixel_size, $
    X_FID=proj_info.nav.fid, X_POS=proj_info.nav.lon_band, Y_FID=proj_info.nav.fid, Y_POS=proj_info.nav.lat_band
  if (glt_fid[0] eq -1) then begin
    remove_tmpglt, glt_fid, tmp_glt_name
    return, {status: -1, msg: 'Lookup table generation failed.'}
  endif

  ; Apply the look up table:
  ENVI_DOIT, 'ENVI_GEOREF_FROM_GLT_DOIT',FID=proj_info.image.fid, $
    GLT_FID=glt_fid,pos=proj_info.image.bands, background=65535., $
    out_name=proj_info.output.outfile, r_fid=result_fid
  
  outputRaster = ENVIFIDToRaster(result_fid)
  
  ; Amend the file's metadata
  if (outputRaster.metadata.hastag ('data ignore value')) then begin
    outputRaster.metadata.updateitem, 'data ignore value', 65535.0
  endif else begin
    outputRaster.metadata.additem, 'data ignore value', 65535.0
  endelse
  outputRaster.writemetadata
  
  ; Unnecessary to explicitly display results, since
  ; ENVI_GEOREF_FROM_GLT_DOIT displays its output automatically
  
  ; Clean up temp GLT:
  remove_tmpglt, glt_fid, tmp_glt_name

  return, projstat
end
