; Procedure to perform volcano scan atmospheric correction to OMEGA data.
; Input is file(s) created by readomega.
;
; INPUT:
;   corr_info.c_in.fid       C-channel input file ID
;   corr_info.c_in.filename  C-channel input filename
;   corr_info.l_in.fid       L-channel input file ID
;   corr_info.l_in.filename  L-channel input filename
;   corr_info.c_out.outfile  C-channel output filename
;   corr_info.l_out.outfile  L-channel output filename

function atmospheric_correction, corr_info, PROGRESS_BAR=progress_bar

COMPILE_OPT IDL2

FORWARD_FUNCTION AUTO_WID_MNG, ENVIRASTERTOFID, WIDGET_AUTO_BASE, ENVIAbortable

aux_location = oat_addon_location(subdir='readomega_aux')

; progress bar setup
if keyword_set(progress_bar) then begin
  e = ENVI(/CURRENT)
  Channel = e.GetBroadcastChannel()
  Abort = ENVIAbortable()
  Start = ENVIStartMessage('Raw OMEGA Data to I/F', Abort)
  Channel.Broadcast, Start
  tstart = systime(1)
endif

e = envi(/current)

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Initializing', 0, Abort)
  Channel.Broadcast, Progress
endif

c_raster = ENVIFIDToRaster(corr_info.c_in.fid)
l_raster = ENVIFIDToRaster(corr_info.l_in.fid)

c_data = c_raster.GetData()
l_data = l_raster.GetData()

csz = size(c_data)
lsz = size(l_data)

lines = csz[2] ; num lines
pixels = csz[1] ; num samples
; set up data

; imitate jdat in readomega, except we don't need space for V
jdat = fltarr(csz[1],256,csz[2]) ; #s x (256, C bands + L bands) x #l
jdat[*,0:127,*]   = transpose(c_data,[0,2,1])
jdat[*,128:255,*] = transpose(l_data,[0,2,1])

orbnum_str = file_basename(corr_info.c_in.filename)
orbnum=fix(strmid(orbnum_str,3,4))

atmorap=fltarr(256)
openr, u, aux_location+'atmorap.dat', /get_lun
readf, u, atmorap
free_lun, u


; Do the correction, from Brown readomega edit

;***********************************************
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;ATMOSPHERIC CORRECTION FOR IR BANDS...
; (not done in VNIR b/c no atm bands there)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Apply a simplistic attenuation model.
;Calculate I/F params needed...
;***********************************************

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Performing atmospheric correction', 25, Abort)
  Channel.Broadcast, Progress
endif

j114=reform(jdat[*,114,*])
wz1=where(j114 eq 0)
if (wz1[0] ne -1) then j114[wz1]=1.
pente=reform(jdat[*,26,*]/j114[*,*])
wz2=where(pente eq 0)
if (wz2[0] ne -1) then pente[wz2]=1.

if (orbnum lt 1900) then begin
  j68=reform(jdat[*,68,*])
  wz3=where(j68 eq 0)
  if (wz3[0] ne -1) then j68[wz3]=1.
  arg=reform(jdat[*,76,*]/j68[*,*])
  wz4=where(arg eq 0)
  if (wz4[0] ne -1) then arg[wz4]=1.
  alti=reform(alog(arg)+alog(pente)*0.0909)
endif else begin
  j70=reform(jdat[*,70,*])
  wz3=where(j70 eq 0)
  if (wz3[0] ne -1) then j70[wz3]=1.
  arg=reform(jdat[*,76,*]/j70[*,*])
  wz4=where(arg eq 0)
  if (wz4[0] ne -1) then arg[wz4]=1.
  alti=reform(alog(arg)+alog(pente)*0.06815)
endelse
if (wz1[0] ne -1) then begin & pente[wz1]=0. & alti[wz1]=0. & endif
if (wz2[0] ne -1) then begin & pente[wz2]=0. & alti[wz2]=0. & endif
if (wz3[0] ne -1) then alti[wz3]=0.
if (wz4[0] ne -1) then alti[wz4]=0.


if (orbnum lt 1900) then expon=alti/alog(atmorap[76]/atmorap[68]) else expon=alti/alog(atmorap[76]/atmorap[70])

for n=0,lines-1 do begin
  for i=0,pixels-1 do begin
    jdat[i,0:255,n]=jdat[i,0:255,n]/(atmorap^expon[i,n])
  endfor
endfor

; Save stuff, as appropriate
; 
; get back to samples by lines by bands cubes for the two sensors
c_corrected = transpose(jdat[*,0:127,*],[0,2,1])
l_corrected = transpose(jdat[*,128:255,*],[0,2,1])

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Writing outputs', 60, Abort)
  Channel.Broadcast, Progress
endif

c_data_out = ENVIRaster(INHERITS_FROM=c_raster, URI=corr_info.c_out.outfile)
c_data_out.SetData, c_corrected
c_data_out.save
display_raster_current_view, c_data_out

l_data_out = ENVIRaster(INHERITS_FROM=l_raster,URI=corr_info.l_out.outfile)
l_data_out.SetData, l_corrected
l_data_out.save
display_raster_current_view, l_data_out

if keyword_set(progress_bar) then begin
  Finish = ENVIFinishMessage(Abort)
  Channel.Broadcast, Finish
  tnow = systime(1)
  print, 'Atmosphere correction time (s): ',tnow-tstart
endif

print, "Atmospheric_correction has finished successfully"
; Success status code
return, {status: 0, msg:''}

end